# This is Control Panel script for Altera Cyclone III Starter Board
# 
package require Tk


#---------------------Procedures-----------------------------------------
#
#  Procedure to enable/disable buttons
proc buttons_control {} {
global connected
  if {!$connected} {
  # If device is not connected all the buttons need to be disabled
    .leds.btns.set_button configure -state disabled
    .leds.btns.select_button configure -state disabled
    .leds.btns.unselect_button configure -state disabled
    .ssram.f3.write configure -state disabled
    .ssram.f3.read configure -state disabled
    } else {
  # Otherwise enable all the buttons
    .leds.btns.set_button configure -state normal
    .leds.btns.select_button configure -state normal
    .leds.btns.unselect_button configure -state normal
    .ssram.f3.write configure -state normal
    .ssram.f3.read configure -state normal
    }

}
# End of proc buttons_control
#
#
# Procedure to connect Altera device
proc device_connect {} {
global connected
global usb
global device
global status_bar
if { [catch {set usb [lindex [get_hardware_names] 0]}]} {
     # Something went wrong with USB connection
     set status_bar "USB connection error"
     set connected 0
     buttons_control
     return
   } elseif { [catch {set device [lindex [get_device_names -hardware_name $usb] 0]}]} {
     # Something went wrong with Altera device
     set status_bar "Altera device connection error"
     set connected 0
     buttons_control
     return
   }
 # If we're here - device connected
  set status_bar "Device connected $usb $device"
  set connected 1
  buttons_control
}
# End of proc device_connect
#
# Procedure to control LEDs
proc leds_set {} {
global led1
global led2
global led3
global led4
global usb
global device
global status_bar

set binary $led4$led3$led2$led1

open_device -hardware_name $usb -device_name $device
device_lock -timeout 10000
# Command in Altera chip to set LED's is 1
set ret [catch {device_virtual_ir_shift -instance_index 0 -ir_value 1} ]
if {$ret} {
  # In case VJTAG instance is not created to avoid system error from Tcl
  set status_bar "Error: VJTAG instance not found"
  catch {device_unlock}
  catch {close_device}
  return  

#}
set tmp [device_virtual_dr_shift -instance_index 0 -dr_value $binary -length 4]
catch {device_unlock}
catch {close_device}
set status_bar "Info: Set command done successfully"

}
# End of proc leds_set
#
#
proc ssram_write {} {
global usb
global device
global status_bar
global ssram_mode
global ssram_addr
global ssram_w
global ssram_file_to_write

set ssram_cmd 00
set file ""

open_device -hardware_name $usb -device_name $device
device_lock -timeout 10000
# Command in Altera chip to write SSRAM is 2
if { $ssram_mode != 5 } {
# Write one double word 
  set ret [catch {device_virtual_ir_shift -instance_index 0 -ir_value 2} ]
  if {$ret} {
    # In case VJTAG instance is not created to avoid system error from Tcl
    set status_bar "Error: VJTAG instance not found"
    catch {device_unlock}
    catch {close_device}
    return  

  }
   # Need to specify write parameters based on what needs to be written - dword or byte
   if {$ssram_mode == 0} {
     set ssram_cmd 40
   } elseif {$ssram_mode == 1} {
     set ssram_cmd 60
   } elseif {$ssram_mode == 2} {
     set ssram_cmd 68
   } elseif {$ssram_mode == 3} {
     set ssram_cmd 70
   } elseif {$ssram_mode == 4} {
     set ssram_cmd 78
   }
   device_virtual_dr_shift -instance_index 0 -dr_value   $ssram_w$ssram_cmd$ssram_addr  -length 64 -value_in_hex
   catch {device_unlock}
   catch {close_device}
   set status_bar "Info: Write SSRAM done successfully"


  } else {
  # Sequential write - data taken from the file.
  # File must contain hex data with no spaces. Otherwise tcl will issue error 
  # message during write process
    set ret [catch {set file [open $ssram_file_to_write r]} ]
    if {$ret} {
    # If catch returns non zero value file open error occured.
    # Inform about it and return
      set status_bar "Error: file open error in sequential write operation"
      catch {device_unlock}
      catch {close_device}
      return
       }
    # Round address to doubleword
    set seq_ssram_addr [format %.6X [expr (0x$ssram_addr/4)*4 ]]
    set count 0
    set ssram_cmd 40
    set ret [catch {device_virtual_ir_shift -instance_index 0 -ir_value 2} ]
    if {$ret} {
      # In case VJTAG instance is not created to avoid system error from Tcl
        set status_bar "Error: VJTAG instance not found"
        catch {device_unlock}
        catch {close_device}
        return  
        }

    # Note for sequential write operations address must be aligned with dword - power by 4
    while { [string bytelength  [set data_read [read $file 8] ] ] == 8 } {
         # Data is read from file by 8-byte chunks. If less than 8 bytes left it's truncated
         # 
         # Start address from variable seq_ssram_addr
	 #
            device_virtual_dr_shift -instance_index 0\
            -dr_value $data_read$ssram_cmd$seq_ssram_addr -length 64 -value_in_hex
	    set seq_ssram_addr [format %.6X [expr (0x$seq_ssram_addr + 4)]]
            incr count
           }
    set status_bar "Info: Seq. write done. $count dwords written.\
 Last address - [format %.6X [expr (0x$seq_ssram_addr - 4)]]"
    close $file
  }  
catch {device_unlock}
catch {close_device}
}
# End of ssram_write proc
#
# Procedure to check functionality of bypass circuitry of VJTAG
proc bypass_check {} {

global usb
global device
global status_bar
global test_sq

open_device -hardware_name $usb -device_name $device
device_lock -timeout 10000
# Command in Altera chip to enable bypass register is 0
set ret [catch {device_virtual_ir_shift -instance_index 0 -ir_value 0} ]
if {$ret} {
  # In case VJTAG instance is not created to avoid system error from Tcl
  set status_bar "Error: VJTAG instance not found"
  catch {device_unlock}
  catch {close_device}
  return  

}
set tmp [device_virtual_dr_shift -instance_index 0 -dr_value $test_sq -length 32 \
-value_in_hex]
catch {device_unlock}
catch {close_device}
set status_bar "Info: Data received - $tmp"


}
# End of bypass_check proc


# Procedure to read SSRAM
proc ssram_read {} {
global usb
global device
global status_bar
global ssram_mode
global ssram_addr
global ssram_w
global ssram_r
global ssram_file_to_read

set ssram_cmd 00

open_device -hardware_name $usb -device_name $device
device_lock -timeout 10000
# Command in Altera chip to read SSRAM is 3
set ret [catch {device_virtual_ir_shift -instance_index 0 -ir_value 3} ]
if {$ret} {
  # In case VJTAG instance is not created to avoid system error from Tcl
  set status_bar "Error: VJTAG instance not found"
  catch {device_unlock}
  catch {close_device}
  return  

}
if {$ssram_mode != 5} {
  # Set parameters of read command
  device_virtual_dr_shift -instance_index 0 -dr_value DDDDDDDD$ssram_cmd$ssram_addr -length 64 -value_in_hex
  # Do actual read operation
  device_virtual_ir_shift -instance_index 0 -ir_value 4
  set ssram_r [device_virtual_dr_shift -instance_index 0 -length 32 -value_in_hex]
  set status_bar "Info: Read SSRAM done successfully"
  } else {
  # Read data to file in this case.
  # Number of dwords to read is in the field ssram_r
  set seq_ssram_addr [format %.6X [expr (0x$ssram_addr/4)*4 ]]

  set ret [catch {set file [open $ssram_file_to_read w]} ]
  if {$ret} {
  # If catch returns non zero value file open error occured.
  # Inform about it and return
    set status_bar "Error: file open error in sequential read operation"
    catch {device_unlock}
    catch {close_device}
    return
     } 
  if {$ssram_r > 0xf000} {
      set status_bar "Error: Dwords to be read is greater F0000"
      catch {device_unlock}
      catch {close_device}
      close $file
      return
     }
  for {set i 1} {$i <= $ssram_r} {incr i 1} {
      # Set parameters for read operation
      device_virtual_ir_shift -instance_index 0 -ir_value 3
      device_virtual_dr_shift -instance_index 0 -dr_value DDDDDDDD$ssram_cmd$seq_ssram_addr -length 64 -value_in_hex
      # Do actual read operation
      device_virtual_ir_shift -instance_index 0 -ir_value 4
      set tmp [device_virtual_dr_shift -instance_index 0 -length 32 -value_in_hex]
      puts $file $tmp
      set seq_ssram_addr [format %.6X [expr (0x$seq_ssram_addr + 4)]]
     } 
    set status_bar "Info: Sequential read done. [expr ($i-1)] dwords read"
    close $file

  }
  catch {device_unlock}
  catch {close_device}

}
# End of ssram_read proc
#
# Procedure to open dialog window to read/write a file
proc fileDialog {w ent operation} {
global ssram_file_to_read
global ssram_file_to_write

  # Type names 		Extention(s)		Mac  File Type(s)
  #
  #-----------------------------------------------------------------
  set types {
    {"Hex files"		{.hex}}
    {"All files"		*}
  }
  if {$operation=="open"} {
    set ssram_file_to_write [tk_getOpenFile -filetypes $types -parent $w]
  } else {
    set ssram_file_to_read [tk_getSaveFile -filetypes $types \
    -parent $w -initialfile template -defaultextension .hex]
  }
}
# End of FileDialog procedure

#-------------------End of procedures------------------------------------
#
#-------------------Global variables-------------------------------------
variable usb ""
variable device ""
variable status_bar "Device not connected"
variable ssram_addr "000000"
variable ssram_w "00FC00FF"
variable data_to_write ""
variable data_to_read ""
variable ssram_file_to_read ""
variable ssram_file_to_write ""

# At the beginning of the routine assume that device is not connected
set connected 0

set ssram_mode 0

set led1 0
set led2 0
set led3 0
set led4 0

#-------------------End of global variables------------------------------
#
# Rename main window
wm title . "DK-START-3C25N Control panel v1.1"
# Position window
wm geometry . +100+100

# Main control buttons
frame .buttons
pack .buttons -side bottom -fill x -pady 2m
button .buttons.close -text Close -command {destroy .} -width 15
button .buttons.connect -text Connect -command {device_connect} -width 15
pack .buttons.close .buttons.connect -side left -expand 1

#Status line
frame .status
pack .status -side bottom -fill x
entry .status.e1 -textvariable status_bar
pack .status.e1 -side left -fill x -padx 5 -expand 1

#------------------LED control frame------------------------
#
set w .leds
labelframe $w -text "LED control" -padx 2 -pady 2
pack $w -side top
set w .leds.check
frame $w
pack $w
# Checkbuttons
checkbutton $w.led1 -text "LED1" -variable led1 
checkbutton $w.led2 -text "LED2" -variable led2
checkbutton $w.led3 -text "LED3" -variable led3
checkbutton $w.led4 -text "LED4" -variable led4
pack $w.led1 $w.led2 $w.led3 $w.led4 -side left 

set w .leds.btns
frame $w
pack $w -pady 5
button $w.set_button -text Set -command {leds_set} -width 10
button $w.select_button -text "Select All" -command {set led1 1; set led2 1; set led3 1\
; set led4 1} -width 10
button $w.unselect_button -text "Unselect All" -command {set led1 0; set led2 0; set led3 0\
; set led4 0} -width 10
pack $w.set_button $w.select_button $w.unselect_button -side left -padx 2
#
#-------------End of LED control frame-----------------------------------
#
#
#-------------SSRAM Control frame----------------------------------------
#
set w .ssram
labelframe $w -text "SSRAM control (Size: 80000h words)" -padx 2 -pady 10
pack $w -side top

set w .ssram.f1
frame $w
pack $w 
label $w.lab1 -text "Address"
label $w.lab2 -text "wDATA"
label $w.lab3 -text "rDATA"
entry $w.e1 -width 8 -textvariable ssram_addr
entry $w.e2 -width 12 -textvariable ssram_w
entry $w.e3 -width 12 -textvariable ssram_r
pack $w.lab1 $w.e1 $w.lab2 $w.e2 $w.lab3 $w.e3 -side left -padx 5

set w .ssram.f2
frame $w
pack $w -pady 5
# Variable ssram_mode controls mode or R/W access to the memory
# ssram_mode = 0 - Read/Write doubleword
# ssram_mode = 1...4 - Read/Write byte 0...3
# ssram_mode = 5 - Sequential access

radiobutton $w.dword -text "Doubleword" -variable ssram_mode -value 0
radiobutton $w.b0 -text "Byte 0" -variable ssram_mode -value 1
radiobutton $w.b1 -text "Byte 1" -variable ssram_mode -value 2
radiobutton $w.b2 -text "Byte 2" -variable ssram_mode -value 3
radiobutton $w.b3 -text "Byte 3" -variable ssram_mode -value 4
radiobutton $w.swrite -text "Sequential Access" -variable ssram_mode -value 5
pack $w.dword $w.b0 $w.b1 $w.b2 $w.b3 $w.swrite -side left

set w .ssram.f4
frame $w
pack $w -pady 5
# File to be written to SSRAM - must exist. We just open it
label $w.lab1 -text "File to write"
entry $w.e1 -width 45 -textvariable ssram_file_to_write
button $w.btn1 -width 10 -text "Browse" -command {fileDialog . $w.e1 open } 
pack $w.lab1 $w.e1 $w.btn1 -side left -padx 5

set w .ssram.f5
frame $w
pack $w -pady 5
# File to save data read from SSRAM. May exist or created new
label $w.lab1 -text "File to read"
entry $w.e1 -width 45 -textvariable ssram_file_to_read 
button $w.btn1 -width 10 -text "Browse" -command {fileDialog . $w.e1 save}
pack $w.lab1 $w.e1 $w.btn1 -side left -padx 5

set w .ssram.f3
frame $w
pack $w -pady 5
button $w.write -text "Write" -command {ssram_write} -width 15
button $w.read -text "Read" -command {ssram_read} -width 15
pack $w.write $w.read -side left -padx 10

#
#-------------End of SSRAM Control frame---------------------------------
#
# After window has been created disable Altera control buttons
# as device is not connected yet
buttons_control

# This line actually draws window
#
tkwait window .
